import peasy.*;

PeasyCam cam;
PFont labelFont;

// Data containers
Table data;
int rows,cols;
int hiddenCol;
int[][] dataMatrix;
int[] maxList;
int[] minList;
Cellv4[] cellList;

// Layout
final int gridSize = 40;
final int zFactor = 30;
final int satellitesFactor = 30;
final int labelMargin = 5;
final int zOffset = 10;
final float sizeMin = 0.8;
final float sizeMax = 2.5;
final float distMin = 10;
final float distMax = (gridSize/2)+10;
boolean rotationOn = true;
boolean zLinesOn = false;
final float lineHue = 170;

// Labeling/Grid
final int labelColor = 160;
final int gradientMin = 80;
final int gradientMax = 180;

// Table
final int YEAR = 0;
final int MONTH = 1;
final int IS_ECON = 6;

final int MARX = 2;
final int KEYNES = 3;
final int CAPITAL = 4;
final int CRISIS = 5;

// Misc.
final int absMax = 2147483647;


void setup() {

  size(1280,720,OPENGL);
  frameRate(60);
  sphereDetail(1);

  if (frame != null) {
    frame.setResizable(true); 
  }

    //setting up the camera
  cam = new PeasyCam( this,0,0,0,1000);       
  cam.setMinimumDistance(200);
  cam.setMaximumDistance(2000);
  
  smooth();

  // Set font
  labelFont = createFont("Courier New",40);
  
  // Load data
  data = loadTable("assignment4_data.csv", "header");
  rows = data.getRowCount();
  cols = data.getColumnCount();
  
  // Hidden column is the last column
  hiddenCol = cols-1;
  
  // Resize arrays
  dataMatrix = new int[rows][cols];
  maxList = new int[cols];
  minList = new int[cols];
  cellList = new Cellv4[rows];

  // Populate data matrix
  for (int i=0; i<rows; i++) {
    for (int j=0; j<cols; j++) {
      dataMatrix[i][j] = data.getInt(i,j);
    }
  }

  // Prepare minimum and maximum matrices
  for (int i=0; i<rows; i++) {
    for (int j=0; j<cols; j++) {
      maxList[j] = 0;
      minList[j] = absMax;  
    }
  }

  // Populate minimum and maximum matrices
  for (int i=0; i<rows; i++) {
    for (int j=0; j<cols; j++) {
      if (dataMatrix[i][j] < minList[j]) {minList[j] = dataMatrix[i][j];}
      if (dataMatrix[i][j] > maxList[j]) {maxList[j] = dataMatrix[i][j];}
    }
  }

  // Populate and initialize cell list
  for (int i=0; i<rows; i++) {
    
    MinMaxInt v1 = new MinMaxInt(dataMatrix[i][MARX],minList[MARX],maxList[MARX],"Marx");
    MinMaxInt v2 = new MinMaxInt(dataMatrix[i][KEYNES],minList[KEYNES],maxList[KEYNES],"Keynes");
    MinMaxInt v3 = new MinMaxInt(dataMatrix[i][CAPITAL],minList[CAPITAL],maxList[CAPITAL],"Capital");
    MinMaxInt v4 = new MinMaxInt(dataMatrix[i][CRISIS],minList[CRISIS],maxList[CRISIS],"Crisis");

    cellList[i] = new Cellv4(v1,v2,v3,v4);

  }

  // Cut the hidden column
  cols-=1;
    
};


void draw() {

  lights();
  
  background(255,255,250);

  // Draw around origin!
  int w = (8*gridSize);
  int h = (11*gridSize);
 
  for (int i=0; i<rows; i++) {

    // Covert to coordinates
    int x = ((dataMatrix[i][YEAR]-2006)*gridSize)-w/2;
    int y = ((dataMatrix[i][MONTH]-1)*gridSize)-h/2;
    int z = int((dataMatrix[i][IS_ECON]-minList[IS_ECON])/zFactor);

    textFont(labelFont);
    textSize(5);
    fill(labelColor);

    textAlign(CENTER,BOTTOM);
    if(i%12 == 0) {text(dataMatrix[i][YEAR],x,-((h/2)+labelMargin),0);}
    textAlign(RIGHT,CENTER);
    if(i%7 == 0) {text(dataMatrix[i][MONTH],-((w/2)+labelMargin),y,0);}
    
    // Update cells
    cellList[i].updatePos(new PVector(x,y,z));
    cellList[i].updateV1(dataMatrix[i][MARX]); 
    cellList[i].updateV2(dataMatrix[i][KEYNES]);
    cellList[i].updateV3(dataMatrix[i][CAPITAL]); 
    cellList[i].updateV4(dataMatrix[i][CRISIS]);

    cellList[i].draw();
  
  }
  
};


void keyPressed() {
  if (key == 's') {rotationOn=!rotationOn;}
  if (key == 'z') {zLinesOn=!zLinesOn;}
};